;
;	*****************************
;	*                           *
;	*    T i n y   P I L O T    *
;	*         for 6502          *
;	*                           *
;	*     by Nicholas Vrtis     *
;	*            in             *
;	*  MICRO, iss. 16, pg. 41   *
;	*      September, 1979      *
;	*                           *
;	*****************************
;
; ***    This port for Commodore 16    ***
; *** and Plus/4 by Dave Hassler, 2025 ***
;
;    With I, L, and P statement additions
; by Bob Applegate, 1980.
;    X statement addition, Zap command, plus mod to
; L statement, added color and sound to T: statement,
; and title by Dave Hassler, 2023, '25.
;    Scanned source hand-entered and edited for
; clarity and modernity by Dave Hassler.
;    NV's original comments in upper case,
; all others in lower case (mine with DHH).
;    Any mistakes found below are very likely
; my own. --DHH
;

; **** PAGE ZERO REFERENCES - now on page $10 for C16 --dhh


CHRS	= $1003
NAME	= $102B
VARIBS	= $1053
IFLAG	= $1087
HOLDY	= $1088
WORK	= $1089
RESULT	= $108B
ANSX	= $108D
SIGNIF	= $108E
OPRATN	= $108F
NUMDSP	= $1090
RETURN	= $1095
CURAD	= $D8		; free Z-Pg loc for indir-Y
STRING	= $10A6

; Uses $10A0-$10A5 for Bob's ML call variables, if the user
; sets up for the L: statement (variables A-C).
; $10D4-$10DB used by Bob's RND and INPUT statements --dhh

RNUMB	= $DE		; Bob's P stmt - dhh
INTGER	= $10DA		; Bob's I stmt - dhh
NORAD	= $10EC		; Robert's NORAD variable for M:
STORX	= $DA		; <-- these 2 locations used by
STORY	= $DB		; Bob's KIM-1 additions for
			; safe KIM-1 I/O.
ZAPADR	= $DC		; used for indirect Y during clear mem.
COUNTR	= $E4		; loop counter for sound control - dhh
LST	= $10EF
FLG	= $10F1
CR	= $0D
MONITR	= $FF52		; C-16 MON from kernal jump table
TIMER	= $FF05		; TED timer 3 high

* = $1100

; **** PRINT TITLE

	JSR WELCOM	; go print a welcome line --DHH

; **** START OF EDITOR PORTION

START	LDA #$80	; SET MODE TO EDIT FOR PRT ROUTINE
	STA IFLAG
	JSR SETBGN	; SET UP STARTING DATA AREA ADDRESS

; **** HERE IS THE START OF EACH NEW LINE

ELINE	LDA #$3E	; OUTPUT A '>' PROMPT CHARACTER
	JSR SAVOUT

; **** HERE IS WHERE EACH INPUT CHARACTER IS OBTAINED

EGET	JSR SAVIN
	TAX		; CHECK FOR NULLS AND IGNORE
	BEQ EGET	; SO THEY DON'T GET CONFUSED WITH EOL

	CMP #$5E	; IS IT '^'?
	BEQ START	; YES - START AT BEGINNING AGAIN

; **** next 2 are added editor commands - dhh

	CMP 	#$5B	; is it '[' on C-16? (was '~')
	BNE	IS_EXIT	; no, try _
	JMP	ZAPRAM	; yes, go do it

; ** this will need to JMP to basic's MONITOR cmd - dhh

IS_EXIT	CMP 	#$5F		; 'left arrow' (underscore) for quit to MON?
	BNE	IS_AT		; no, try @
	JMP	MONITR		; exit, jump to c16 monitor (can save/load)
; ****

IS_AT	CMP #$40	; IS IT '@' FOR EXECUTE REQUEST?
	BEQ EXEC	; YES, GO START ON THAT

	CMP #$14	; IS IT A BACKSPACE? ('del' for C16)
	BNE TRYDSP	; NO - GO CHECK FOR DISPLAY REQUEST
	
	DEY		; YES - BACK UP ONE CHARACTER
	BPL EGET	; BUT CHECK FOR PAST START OF LINE
	INY		; BACKED UP TOO FAR - DISALLOW
	BPL EGET	; UNCONDITIONAL

TRYDSP	CMP #$5C	; IS IT 'pound' FOR DISPLAY LINE REQUEST?
	BNE TRYREP	; NO - CHECK FOR REPLACEMENT REQUEST

; **** DISPLAY TO THE NEXT CARRIAGE RETURN

	JSR PRT		; PRINT THE LINE
	BCS ELINE	; UNCONDITIONAL

TRYREP	CMP #$25	; IS IT '%' REQUEST TO PAD A LINE?
	BNE CHAR

; **** PAD THE LINE FROM CURRENT LOC TO EOL WITH DELETE CHAR

PADLOP	LDA (CURAD),Y	; GET CURRENT CHARACTER
	BEQ SETNL	; IF ZERO, WE ARE DONE
	LDA #$80	; ELSE MAKE IT A DELETE CHAR (was $ff -dhh)
	STA (CURAD),Y
	INY		; BUMP TO NEXT CHARACTER
	BPL PADLOP	; LOOP IF HAVEN'T DONE 128
	DEY		; LINE IS TOO LONG - BACK UP ONE
	LDA #CR		; FORCE AN EOL HERE

; **** IT WASN'T AN EDIT CHARACTER - MUST BE DATA TO SAVE

CHAR	CMP #CR		; IS IT CARRIAGE RETURN AS EOL?
	BNE CHAR1	; SKIP AHEAD IF NOT
	LDA #$00	; ELSE CONVERT CR TO 00 AS EOL
CHAR1	STA (CURAD),Y	; PUT IT AWAY
	BEQ SETNL	; BRANCH IF YES
	INY		; ELSE BUMP TO SET UP FOR NEXT ONE
	BPL EGET	; AND GO GET IT IF STILL ROOM ON LINE
	DEY		; ELSE POINT BACK TO LAST CHAR & FALL THRU

SETNL	JSR LINEND	; DO CR/LF AND FIX UP CURAD
	BCS ELINE	; GO START A NEW LINE

; **** EXECUTION PORTION STARTS HERE

EXEC	JSR SAVCR	; EXTRA BLANK LINE AFTER EDITOR
RESTRT	JSR SETBGN	; HERE IF FROM J:*
	LDX #$33	; ZERO VARIABLE AREAS
	LDA #$00
	STA RETURN+1	; 
RESTR1	STA VARIBS,X
	DEX
	BPL RESTR1

LSTART	LDA (CURAD),Y	; GET CHARACTER FROM THE LINE
	CMP #$2A	; CHECK FOR '*' LABEL MARKER
	BNE CHKCON	; IF NOT - GO CHECK FOR CONDITIONAL
	INY		; OTHERWISE SKIP PAST THE '*'
SKPNXT	INY		; SKIP PAST THE NEXT CHARACTER
	BNE LSTART	; UNCONDITIONAL

; **** FLAG DEPENDENT PROCESSING HERE

CHKCON	CMP #$59	; CHECK FOR 'Y' REQUEST
	BEQ TFLAG	; BRANCH IF YES
	CMP #$4E	; IF NOT - CHECK FOR 'N' REQUEST
	BNE STRTST	; BRANCH IF NEITHER

; **** SEE IF CONDITIONAL MATCHES FLAG

TFLAG	CMP FLG		; SEE IF THEY MATCH
	BEQ SKPNXT	; SKIP TO NEXT CHAR AND EXECUTE LINE

; **** NO MATCH - SKIP THIS STATEMENT

FWD	JSR FWD1	; USE THIS SUBROUTINE
	BCS LSTART	; UNCONDITIONAL

STRTST	JMP EXTRA	; head to Bob's and my extra statements --dhh
	NOP

; **** ENTER NAME STATEMENT

XQUEST	CMP #$3F	; IS IT '?' FOR ENTER NAME?
	BNE XA		; BRANCH IF NOT
	SEC		; TURN HIGH BIT ORDER ON TO INDICATE
	ROR IFLAG	; PROCESSING NAME COMMAND
	BNE TAKEIN	; NOW USE THE ACCEPT LOGIC

; **** ACCEPT STATEMENT

XA	CMP #$41	; SEE IF WE HAVE AN 'A'CCEPT STATEMENT
	BNE XC		; BRANCH IF NOT
	LDA CURAD	; SAVE ADDRESS OF THE 'A' STATEMENT
	STA LST		; NOTE: WILL INCLUDE CONDITIONALS
	LDA CURAD+1	; 
	STA LST+1	; 

TAKEIN	LDA #$3F	; DISPLAY '?' PROMPTING CHARACTER
	JSR SAVOUT

	LDX #$27	; CHRS GET STORED BACKWARD
ACHR	JSR SAVIN	; GET AN INPUT CHARACTER
	CMP #$14	; IS IT A BACKSPACE? ('del' on C16 -dhh)
	BNE ACHR1	; BRANCH IF NOT
	INX		; ELSE FORGET ABOUT LAST CHARACTER
	BNE ACHR	; UNCONDITIONAL
ACHR1	CMP #CR		; WAS IT A CARRIAGE RETURN?
	BNE ACHR2	; NO - SKIP AHEAD
	LDA #$00	; YES - CONVERT CR TO EOL
ACHR2	STA CHRS,X	; AND SAVE IT FOR MATCH STATEMENT
	BIT IFLAG	; SEE IF GETTING NAME FIELD
	BPL ACHR3	; BRANCH IF NOT
	STA NAME,X	; ELSE SAVE IN NAME FIELD ALSO
ACHR3	CMP #$00	; IS IT DONE YET?
	BEQ ADONE	; BRANCH IF HE HAS SIGNALED END
	DEX		; ELSE BUMP FOR NEXT INPUT
	BPL ACHR	; AND GO GET IT IF ROOM STILL LEFT
ADONE	JSR SAVCR	; DO A CR/LF TO LET GUY KNOW
	JMP FWD

; **** COMPUTE STATEMENT

XC	CMP #$43	; IS IT 'C' FOR COMPUTE?
	BEQ XC1		; BRANCH IF IT IS
	JMP XM		; ELSE LONG JUMP TO TEST FOR 'M'
XC1	JSR GETIDX	; GET INDEX POINTER TO RESULT
	STX ANSX	; SAVE IT FOR NOW
	LDA #$00	; CLEAR RESULT
	STA RESULT
	STA RESULT+1	; 
	INY		; POINT TO '='
	LDX #$2B	; SET 1ST OPERATION TO '+' FOR ADD
	BNE OPWRAP	; GO SAVE AND SET UP WORK AREA

; **** LOOP FOR EACH NEW CHARACTER IN COMPUTE PROCESS

CMPLOP	INY		; BUMP TO NEXT CHARACTER
	LDA (CURAD),Y	; GET A CHARACTER
	BMI ISOPR	; MINUS IS DELETE/ALSO LAST 'OPERATOR'
	CMP #$2F	; IS IT < '/' FOR AN OPERATION SPECIFIED?
	BCC ISOPR	; BRANCH IF YES
	CMP #$3A	; IF NOT - IS IT > ':' FOR not A NUMBER?
	BCS NOTNMB	; BRANCH IF NOT - MUST BE A VARIABLE

	AND #$0F	; CONVERT NUMBER TO BINARY
	ROR		; SPIN TO HIGH ORDER PART OF A
	ROR
	ROR
	ROR		; LEAVE BIT 3 IN CARRY
	LDX #$04	; 4 BITS TO ROLL INTO WORK
BITROL	ROL WORK+1	; RIPPLE CARRY INTO WORK
	ROL WORK	; FOR 16 BITS
	ASL		; PUT NEXT BIT INTO CARRY
	DEX		; COUNT ONE JUST DONE
	BNE BITROL	; CONTINUE IF MORE TO GO
	BEQ CMPLOP	; ELSE GET NEXT CHARACTER (DIGITS)

NOTNMB	JSR VTRANS	; TRANSFER VARIABLE TO WORK AREA
	JMP CMPLOP	; GO GET NEXT CHARACTER (OPERATION?)

; **** GOT AN OPERATION - FIRST PERFORM PREVIOUS REQUEST

ISOPR	SED		; SET TO DECIMAL MODE
	TAX		; SAVE NEW OPERATION IN X FOR NOW
	LDA OPRATN	; GET PREVIOUS OPERATION
	CMP #$2D	; WAS IT A '-' FOR SUBTRACT?
	BEQ OPMNUS	; BRANCH IF YES
	CLC		; ALL OTHERS ASSUME IT IS ADD
	LDA WORK+1
	ADC RESULT+1
	STA RESULT+1
	LDA WORK
	ADC RESULT
	STA RESULT
	JMP OPWRAP	; GO WRAP UP THE OPERATION

OPMNUS	SEC		; SUBTRACTION
	LDA RESULT+1
	SBC WORK+1
	STA RESULT+1
	LDA RESULT
	SBC WORK
	STA RESULT

OPWRAP	CLD		; GET OUT OF DECIMAL MODE
	STX OPRATN	; SAVE NEW OPERATION
	TXA		; DO TRANSFER TO CHECK FOR '00'/'FF'
	BEQ CMPDON	; DONE IF IT WAS ZERO (EOL)
	BMI CMPDON	; OR DELETE CHARACTERS (FROM FILLING)

	LDA #$00	; ELSE CLEAR WORK AREA FOR NEXT ONE
	STA WORK
	STA WORK+1
	BEQ CMPLOP	; AND GO DO NEXT CHARACTER

CMPDON	LDX ANSX	; GET INDEX TO RESULT
	BPL TOVRIB	; PLUS IS NORMAL INDEX TO A VARIABLE
	
	LDX #$38	; ELSE FUDGE INDEX FOR 'FROM' RESULT
			; USING 'RESULT - VARIBS'
	JSR VTRANS+3	; MOVE RESULT TO WORK AREA
	JSR CNVDSP+3	; CONVERT IT TO DISPLAY FORM
	LDX #$04	; TRANSFER DISPLAY TO NUMBER AREA
TALOOP	LDA NUMDSP,X
	STA CHRS+35,X	; NOTE OFFSET TO PUT IT AT THE END
	DEX		; (was '23' in orig. source, no '$'
	BPL TALOOP	;           to denote hex no. -DHH)
	BMI XFWD	; UNCONDITIONAL
TOVRIB	LDA RESULT+1	; DESIRED VARIABLE
	STA VARIBS+1,X
	LDA RESULT
	STA VARIBS,X
XFWD	JMP FWD		; AND GO DO THE NEXT ONE

; **** PROCESS MATCH STATEMENT

XM	CMP #$4D	; IS IT 'M' FOR MATCH?
	BNE XU		; BRANCH IF NOT 
MDEY	DEY		; BACK UP ONE FOR WHAT FOLLOWS
MCHKX	INY		; POINT TO MATCH CHARACTER
	LDX #$27	; START AT FIRST ACCEPTED CHARACTER
MCHK	LDA (CURAD),Y	; GET THE MATCH CHARACTER
	BEQ MXY		; THEY HAVE MATCHED TO END OF 'M:' STMT
	CMP CHRS,X	; CHECK FOR MATCH
	BNE MXNMCH	; BRANCH IF MATCH FAILED
	INY		; ELSE BUMP TO NEXT PAIR OF CHARACTERS
	DEX
	BPL MCHK	; AND GO CHECK IF STILL DATA LEFT
MXY	LDX #$59	; BOTH EQUAL - SET FLAG TO 'Y'
	BNE MX		; UNCONDITIONAL
MXNMCH	CMP #$24	; IS IT '$' FOR VARIABLE REQUEST?
	BEQ MNUMB	; YES - MATCH TO NUMERIC VARIABLE
	CMP #$2C	; IS IT A COMMA GROUP SEPARATOR?
	BEQ MXY		; YES - MATCHED SO FAR - SET IT AS YES

MCOMMA	INY		; NO - SO NEED TO SKIP AHEAD TO COMMA
	LDA (CURAD),Y
	BEQ MXSETN	; IF TO EOL, THERE IS NO MORE TO CHECK
	CMP #$2C	; IS IT A COMMA CHARACTER?
	BEQ MCHKX	; RESTART COMPARE AT NEXT MATCH CHARACTER
	BNE MCOMMA	; LOOP IN SEARCH OF A COMMA
MCOMX	LDY HOLDY	; RESET Y TO CURRENT LINE POINTER
	BNE MCOMMA	; AND GO LOOK FOR NEXT COMMA

MNUMB	INY		; VARIABLE - BUMP TO VARIABLE ID
	STX ANSX	; SAVE CURRENT X FOR NOW
	JSR CNVDSP	; CONVERT VARIABLE TO DISPLAY FORM
	LDX ANSX	; GET POINTER TO INPUT BACK
	STY HOLDY	; HOLD CURRENT 'Y' POINTER
	LDY #$04	; HAVE TO SEARCH UP TO 5 BYTES

MXNOLP	LDA NUMDSP,Y	; GET ONE NUMERIC CHARACTER
	BEQ MXDIFF	; BRANCH IF END - MIGHT BE MATCH
	CMP CHRS,X	; ELSE CHECK AGAINST INPUT
	BNE MCOMX	; BRANCH IF NO MATCH
	DEX		; ELSE CONTINUE MATCHING
	DEY
	BPL MXNOLP	; UNCONDITIONAL

MXDIFF	LDY HOLDY	; RESET Y TO CURRENT LINE POINTER
	INY		; BUMP TO CHARACTER AFTER VARIABLE
	BNE MCHK	; UNCONDITIONAL CONTINUE CHECKING

MXSETN	LDX #$4E	; GET 'N' - MATCH WAS UNSUCCESSFUL
MX	STX FLG		; STORE IT
	BNE XFWD	; UNCONDITIONAL FORWARD TO NEXT LINE

; **** PROCESS USE SUBROUTINE STATEMENT

XU	CMP #$55	; IS IT A 'U' FOR USE SUBROUTINE?
	BNE XJ		; BRANCH IF NOT
	LDA (CURAD),Y	; GET DESTINATION
	PHA		; SAVE THE LABEL CHARACTER
	JSR FWD1	; MOVE TO START OF NEXT LINE
	LDA CURAD
	STA RETURN	; SAVE FOR RETURN ADDRESS
	LDA CURAD+1
	STA RETURN+1
	PLA		; GET DESTINATION BACK
	BNE JDO		; NO GO HANDLE AS JUMP STATEMENT

; **** PROCESS JUMP STATEMENT

XJ	CMP #$4A	; IS IT 'J' FOR MATCH STATEMENT?
	BNE XS		; BRANCH IF NOT
	LDA (CURAD),Y	; GET DESTINATION

JDO	STA IFLAG	; SAVE LABEL CHARACTER
	CMP #$2A	; HAVE '*' TO REQUEST RETURN TO BEGINNING?
	BEQ IREST	; BRANCH IF SO
	CMP #$41	; SEE IF A LABELED JUMP
	BNE JF		; IF NOT 'A', IT'S A NORMAL JUMP

	LDA LST		; ELSE SET TO START OF LAST ACCEPT
	STA CURAD
	LDA LST+1
	STA CURAD+1
	BNE ILNEXT	; UNCONDITIONAL

JF	JSR SETBGN+3	; AND GET BACK TO START OF PROGRAM

FNDMRK	LDA (CURAD),Y	; GET FIRST CHARACTER
	CMP #$2A	; IS IT '*' FOR A MARKER?
	BNE FMNEXT	; NOPE - GO AHEAD TO NEXT LINE
	INY		; ELSE BUMP TO MARKER CHARACTER
	LDA (CURAD),Y	; GET LABEL
	CMP IFLAG	; SEE IF IT'S THE ONE WE WANT
	BEQ ILNEXT	; YES - GO EXECUTE IT
FMNEXT	JSR FWD1	; ELSE GO TO NEXT LINE
	BCS FNDMRK	; AND CONTINUE LOOKING
IREST	JMP RESTRT	; INDIRECT TO RESTRT

; **** STOP STATEMENT

XS	CMP #$53	; IS IT AN 'S' FOR STOP STATEMENT?
	BNE XE		; BRANCH IF NOT
	JMP START	; YES, RETURN TO EDITOR START

; **** EXIT FROM SUBROUTINE

XE	CMP #$45	; IS IT AN 'E'?
	BNE XR		; BRANCH IF NOT
	LDA RETURN+1	; MOVE RETURN ADDRESS TO CURAD
	BEQ XXFWD	; SKIP LINE IF NOT SET
	STA CURAD+1
	LDA RETURN
	STA CURAD
	LDA #$00	; NOW SET TO NOT USED AGAIN
	STA RETURN+1
	BEQ ILNEXT	; UNCONDITIONAL

; **** REMARK STATEMENT

XR	CMP #$52	; IS IT AN 'R'?
	BNE XT		; BRANCH IF NOT - ELSE SKIP THE LINE
XXFWD	JMP FWD		; CAN'T BRANCH THAT FAR ALONE

; **** TYPE STATEMENTS AND SYNTAX ERRORS

XT	CMP #$54	; IS IT A VALID 'T' STATEMENT?
	BEQ TE		; BRANCH IF SO
	DEY		; ELSE BACK UP TO ORIGINAL START
	DEY
TE	JSR PRT		; NOW PRINT THE LINE
ILNEXT	JSR SKPJNK	; CURAD IS SET - SKIP OVER LEADING JUNK
	JMP LSTART	; AND GO START ON THE LINE

; **** PRINT A LINE FROM CURRENT LOCATION TO
; **** NEXT EOL AND THEN SET UP FOR NEXT LINE

PRT	LDA (CURAD),Y	; GET THE CURRENT CHARACTER
	BEQ LINEND	; BRANCH IF TO END OF LINE
	BIT IFLAG	; SEE IF IN EDITOR
	BMI CHROUT	; IF SO, DON'T LOOK FOR '$' (or '&' or ']' --dhh) 

	CMP #$5D	; is it a special sound marker ']'?
	BNE IS_CLR	; no, check for color change
	JMP DO_SND	; yes, go make a noise

IS_CLR	CMP #$26	; is it special change color marker '&'?
	BEQ DOCOLR	; yes, go change text color

	CMP #$24	; IS IT A SPECIAL ONE ('$')?
	BNE CHROUT	; BRANCH IF NOT
	INY		; ELSE BUMP TO NEXT ONE
	LDA (CURAD),Y	; GET VARIABLE
	CMP #$3F	; IS IT REQUEST FOR NAME ('?')?
			; NV source in MICRO shows a ('$') in
			; the comment column, but $3F is in the
			; hex dump and the assembly listing. --dhh
	BEQ NAMEO	; BRANCH IF YES
	JSR CNVDSP	; CONVERT VARIABLE TO DISPLAY
	LDX #$04	; GOT 5 BYTES POSSIBLE

VBDISP	LDA NUMDSP,X	; GET A CHARACTER
	BEQ CHROUT+3	; BRANCH IF TO END OF VARIABLE
	JSR SAVOUT	; ELSE OUTPUT IT
	DEX		; AND COUNT IT
	BPL VBDISP	; UNCONDITIONAL LOOP

NAMEO	LDX #$27	; REMEMBER - IT CAME IN BACKWARDS
	LDA NAME,X
	BEQ CHROUT+3	; BRANCH IF TO END OF NAME
	JSR SAVOUT
	DEX
	BPL NAMEO+2	; UNCONDITIONAL

CHROUT	JSR SAVOUT
NOCHAR	INY
	BPL PRT		; LOOP IF NOT TOO MANY
	BMI SETBGN	; RESET TO BEGINNING IF PAST THE END
LINEND	JSR SAVCR	; OUTPUT CR AND LF

; **** ENTER HERE TO SKIP A LINE WITHOUT PRINT
; **** AND INITIALIZE FOR THE NEXT LINE

FWD1	LDA (CURAD),Y	; GET A CHARACTER
	BEQ SCURAD	; BRANCH IF END OF LINE
	INY		; ELSE BUMP TO NEXT ONE
	BPL FWD1	; LOOP IF NOT TOO MANY
	BMI SETBGN	; RESET TO BEGINNING IF PAST THE END

DOCOLR	INY		; BUMP TO NEXT CHAR
	LDA (CURAD),Y	; get color 'code' (1-8)
	STX STORX	; save X
	CMP #$30	; '0'
	BCC CHROUT	; <0, not number, so output it
	CMP #$3A	; ':'
	BCS CHROUT	; >= ':', so not a number, go print
	SEC
	SBC #$30	; conv to # for index
	TAX		; put color/clr number in X
	LDA COLORS,X	; get PETSCII for the chosen color/switch
	LDX STORX	; get original X back
	JMP CHROUT	; uncond. branch

DO_SND	JSR SOUND	; go process the sound code
	JMP NOCHAR	; when back, don't print anything, but +1 Y

; **** HERE FIXES UP CURAD TO POINT TO BEGINNING OF A LINE
; **** CURAD SHOULD INDEX END OF LINE (WITH Y) ON ENTRY

SCURAD	INY		; BUMP PAST THE CR
	TYA		; MOVE COUNT TO A
	CLC		; CLEAR CARRY FOR ADD
	ADC CURAD	; ADD TO LOW ORDER FIRST
	STA CURAD	; AND SAVE RESULT
	BCC SKPJNK	; SKIP IF NO CARRY FORWARD
	INC CURAD+1	; ELSE BUMP HIGH ORDER

; **** HERE TO SKIP PAST LEADING JUNK ON A LINE

SKPJNK	LDY #$FF	; SET UP Y THIS WAY
SJLOOP	INY		; INCREMENT TO NEXT CHARACTER
	BIT IFLAG	; SEE IF IN EDIT MODE
	BMI SJRTS	; DON'T TRY SKIPPING JUNK IF SO
	LDA (CURAD),Y	; GET CHARACTER TO LOOK AT
	BMI SJLOOP	; IGNORE DELETE CHARACTER ALSO
	CMP #$2A	; LOOK FOR '*' LABEL MARKER
	BEQ SJRTS	; RETURN IF FOUND
	CMP #$3F	; LOOK FOR POSSIBLE OPERATION CHARACTER
	BCC SJLOOP	; CONTINUE SKIPPING IF TOO LOW
SJRTS	SEC		; SET CARRY FOR BRANCHES AFTER RETURN
	RTS		; AND BEFORE RETURN

; **** SET UP BEGINNING ADDRESS OF USER AREA

SETBGN	JSR SAVCR	; START ON A NEW LINE
	LDY #$00	; EVEN PAGE BOUNDARY
	STY CURAD
	STY LST		; ALSO SET UP THIS GUY AS DEFAULT
	LDA #$17	; was $05. Changed to add room for 
	STA CURAD+1	; all the extensions and C16---dhh
	STA LST+1
	BNE SKPJNK	; UNCONDITIONAL

; **** COMPUTE INDEX FOR A VARIABLE

GETIDX	LDA (CURAD),Y	; GET VARIABLE LETTER
	SEC
	SBC #$41	; SUBTRACT 'A' TO MAKE RELATIVE TO ZERO
	ASL		; TIMES TWO BYTES PER VARIABLE
	TAX		; MOVE TO INDEX REGISTER
	RTS		; AND RETURN

; **** TRANSFER A VARIABLE'S DATA TO WORK AREA

VTRANS	JSR GETIDX	; GET INDEX POINTER FIRST
	LDA VARIBS+1,X	; NOW MOVE TO WORK AREA
	STA WORK+1
	LDA VARIBS,X
	STA WORK
	RTS

; **** CONVERT A VARIABLE TO DISPLAY FORM

CNVDSP	JSR VTRANS	; MOVE TO WORK AREA
	BPL ISPLUS	; BRANCH IF POSITIVE
	LDA #$2D	; ELSE PUT IN MINUS SIGN
	STA NUMDSP+4
	SED		; SET DECIMAL MODE INDICATOR
	SEC
	LDA #$00	; SUBTRACT FROM ZERO TO COMPLIMENT

	SBC WORK+1
	STA WORK+1
	LDA #$00
	SBC WORK
	STA WORK
	CLD		; CLEAR DECIMAL MODE
	LDX #$03	; ONLY 4 POSITIONS LEFT
	BNE ISPL1	; SKIP INDEX SET (orig source has BNE ISPL
			; but hex dump shows a branch of 02 --dhh)

ISPLUS	LDX #$04	; PLUS HAS FIVE POSITIONS AVAILABLE
ISPL1	CLC		; TURN OFF SIGNIFICANCE INDICATOR
	ROR SIGNIF
	LDA WORK	; GET FIRST DIGIT
	JSR TOOUT	; PUT TO OUTPUT AREA
	LDA WORK+1	; SECOND DIGIT IS HIGH ORDER OF THIS
	LSR
	LSR
	LSR
	LSR
	JSR TOOUT
	LDA WORK+1	; LOW ORDER IS THIRD DIGIT
	JSR TOOUT
	BIT SIGNIF	; SEE IF IT HAD ANY SIGNIFICANT CHARS
	BMI ISPL2	; SKIP NEXT IF YES
	DEX		; ELSE KEEP THE LAST ZERO THERE
ISPL2	LDA #$00	; INSERT END OF LINE MARKER
	STA NUMDSP,X
	RTS		; AND RETURN

; **** CONVERT CURRENT VALUE TO ASCII AND PUT TO OUTPUT AREA

TOOUT	AND #$0F	; KEEP ONLY LOW ORDER
	ORA #$30	; MAKE IT ASCII
	STA NUMDSP,X	; SAVE REGARDLESS
	BIT SIGNIF	; SEE IF SIGNIFICANCE STARTED
	BMI SETSIG	; YES - ALL ARE IMPORTANT NOW
	CMP #$30	; ELSE SEE IF SHOULD START NOW
	BNE SETSIG	; IMPORTANT IF NOT ZERO
	RTS		; ELSE RETURN

SETSIG	SEC		; SET SIGNIFICANCE BIT ON
	ROR SIGNIF	; ALWAYS
	DEX		; AND POINT TO NEXT AVAILABLE POINTER
PGMEND	RTS		; AND THEN RETURN

; **** was Bob Applegate's KIM-1 specific IN/OUT/CRLF routines
; **** now for CBM kernal jump table --dhh

SAVIN	JSR $FFCF	; CHRIN c16 kernal jump table addr
	RTS
SAVOUT	JSR $FFD2	; CHROUT c16 kernal jump table addr
	RTS
SAVCR	LDA #$0D	; cbm combined cr/lf
	JSR $FFD2	; output it
	RTS

; **** New Tiny PILOT statements added by Bob Applegate.
; **** Appeared in MICRO, Aug. 1980, issue 27, pg. 21
; **** 
; **** I:x -- Input a positive number into any variable x
; **** P:x -- Puts a random number 0-99 into variable x
; **** L:x -- Call ML subroutine loc in variable x (A-C). 
;             Starting addr of variable to be PRE-STORED as:
;		NAME	Addrs
;		A	$10A0, $10A1
;		B	$10A2, $10A3
;		C	$10A4, $10A5

EXTRA	STA IFLAG	; clear high bit for editor -- BA
	INY		; point to ':'
	INY		; and the next character

; **** CALL ML SUBROUTINE STATEMENT
; **** Modified by DHH to remove self-modifying code.

	CMP #$4C	; is it the L command
	BNE XP		; no, go to P command
XL	JSR GETIDX	; compute the index
	LDA $10A0,X	; get the low-order byte from table
	STA STORX	; and store in a temp loc.
	LDA $10A1,X	; get the high-order byte
	STA STORY	; and store in ajacent temp loc.
	STY HOLDY	; necessary, X too?
STRSUB	JSR GOL		; subroutine will return here
	LDY HOLDY
	JMP FWD		; all done, return to PILOT
GOL	JMP (STORX)	; indirect jump to ML program

; **** GENERATE RANDOM NUMBER STATEMENT

XP	CMP #$50	; is it the P command?
	BNE XI		; nope, go to I command
	JSR GETIDX	; compute the variable index
	STX ANSX	; store the index for now
			; ** following is a slightly modified
			; version of Jim Butterfield's random
			; number routine from FBOK, pg. 172.
	SED		; decimal numbers only, please
	SEC		; carry adds value 1
	LDA RNUMB+1	; last value
	ADC RNUMB+4	; add B + carry
	ADC RNUMB+5	; add C
	STA RNUMB	; new random number
	LDX #$04	; move 5 numbers
XPLOOP	LDA RNUMB,X	; get first number
	STA RNUMB+1,X	; move over 1
	DEX		; next number
	BPL XPLOOP	; all moved?
	CLD		; everything back to normal
	LDX ANSX	; pick up the index
	STA VARIBS+1,X	; store the number at variable
			; (should this be just VARIBS? --dhh)
	JMP FWD		; back to PILOT

; **** INPUT A POSITIVE NUMBER STATEMENT

XI	CMP #$49	; see if it's the I command
	BNE XX		; no, go see if X command
	LDA #$3F	; get a '?'
	JSR SAVOUT	; output it as a prompt
	LDA #$00	; clear things out
	STA INTGER	; especially this temp variable
	STA INTGER+1	; ditto
XIINPT	JSR SAVIN	; get input
	CMP #CR		; cr?
	BEQ XINDX	; yes, branch to get index
	SEC		; get ready to subtract
	SBC #$30	; turn ASCII into BCD
	LDX #$04	; get ready to multiply by 10
XILOOP	CLC		; clear things first ('CMC' mnemonic in source)
	ASL INTGER+1	; multiply
	ROL INTGER	; multiply
	DEX		; again?
	BNE XILOOP	; yes, then loop
	CLC		; clear things up again
	ADC INTGER+1	; add the new digit
	STA INTGER+1	; store the answer
	JMP XIINPT	; do it all over again
XINDX	JSR GETIDX	; get the index
	LDA INTGER	; get first part of answer
	STA VARIBS,X	; store it at variable
	LDA INTGER+1	; get the next part
	STA VARIBS+1,X	; and store that
	JSR SAVCR	; start a new line
	JMP FWD		; all done, return to PILOT

; **** X: Statement (swap/transfer NAME and STRING) -- (mine! dhh)

XX	CMP #$58	; see if it's the X: statement
	BNE GOBACK	; no, head back to original Statements
	STY STORY	; save Y 'cuz CURAD needs it
	LDY #$27	; will go 40 times/chars
XXLOOP	LDA NAME,Y	; swappa those strings!
	LDX STRING,Y
	STA STRING,Y
	TXA
	STA NAME,Y
	STA CHRS,Y
	DEY
	BNE XXLOOP
	LDY STORY	; get original Y back
	JMP FWD		; and done

GOBACK	JMP XQUEST	; not a new statement, check old ones


; **** ZAP editor command by Dave Hassler, 2023 (from 6800 Tiny PILOT).
; **** Clears all variables (except Bob's special ML call variables)
; **** and user RAM to $3FFF for that fresh, power-up feeling.
; **** The equivalent of BASIC's 'NEW' command.

ZAPRAM	LDA #$53	;start of variable storage
	STA ZAPADR
	LDA #$10
	STA ZAPADR+1
	LDY #$00
	LDA #$00

ZLOOP	STA (ZAPADR),Y
	INY
	CPY #$33
	BNE ZLOOP

	LDY #$00	; next, user memory
	STY ZAPADR
	LDA #$17	; first page to '00'
	STA ZAPADR+1
MLOOP	LDA #$00
MLOOP2	STA (ZAPADR),Y	; store '00's - the EOL marker
	INY
	BNE MLOOP2	; until Y rolls over
	INC ZAPADR+1	; when it does, +1
	LDA ZAPADR+1	; the page #
	CMP #$40	; is it now Page $40?
	BNE MLOOP	; no, go fill it with '00'

	JMP START	; jump back to START

; **** PLAY SOUNDS ROUTINE IN T: STATEMENT (DHH)

sound	INY		; BUMP TO NEXT CHAR
	LDA (CURAD),Y	; get sound 'code' (ASCII 1-4)
	STX STORX	; SAVE X
	cmp #$31	; is '1'?
	beq bell	; yep, go do 'bell'
	cmp #$32	; is '2'?
	beq blip	; yep, go do 'blip'
	cmp #$33	; is '3'?
	beq bong	; yep, go do 'bong'
	cmp #$34	; is '4'?
	beq buzz	; yep, go do 'buzz'
	jmp chrout	; not 1-4, so print whatever it is

bell	lda #$af	; set up voice 2 for max vol
	sta $ff11	;   but turn off output
	lda #$95	; set note for v2 at $0395
	sta $ff0f
	lda #$03
	sta $ff10
	lda $ff11	; get the voice parameter register
	and #$7f	; strip bit 7, which will turn on v2,
	sta $ff11	; and store it.
	jmp delay	; now for length on time and fade

blip	lda #$a4	; set up voice 2 for softer vol
	sta $ff11	;   but turn off output
	lda #$02	; set note for v2 at $0203
	sta $ff0f
	lda #$03
	sta $ff10
	lda $ff11	; get the voice parameter register
	and #$7f	; strip bit 7, which will turn on v2,
	sta $ff11	; and store it.
	ldx #$0a	; set up the delay/time value
	bne time_a	; and go loop for that long

bong	lda #$af
	sta $ff11
	lda #$a9	; note $00a9. see 'bell' for rest
	sta $ff0f
	lda #$00
	sta $ff10
	lda $ff11
	and #$7f
	sta $ff11
	jmp delay

buzz	lda #$ef	; set up voice 2 for max vol
	sta $ff11	;   and add noise (bit 6)
	lda #$07	; for the rest, see 'blip', above
	sta $ff0f
	lda #$00
	sta $ff10
	lda $ff11
	and #$7f
	sta $ff11
	ldx #$ff
	bne time_a

; *** vol is set to 15 initially to allow for sustain
; *** and decay, as settings 8-15 are all max volume -- dhh

delay	ldx #$14	; set outside loop
init_a	lda #$00	; set inside loop
	sta countr
drop_a	dec countr	; -1 inner counter
	bne drop_a	;   until it's 0
	dex		; -1 the outer counter
	bne init_a	;   and do until 0
	lda $ff11	; load up current volume
	sec		; ready for subtraction
	sbc #$01	; -1
	sta $ff11	; store new volume
	and #$0f	; mask off high nybble
	beq alldun	; if vol=0, then done
	jmp delay	; else, do it all again

time_a	lda #$00	; inside loop
	sta countr
time_b	dec countr	; -1 the inner loop
	bne time_b	; go until 0
	dex		; then -1 the outer loop
	bne time_a	; go until 0 then fall through

alldun	lda #$80	; value to turn off sound and blank
	sta $ff11	;   all parameters.  Store.
	ldx storx	; retrieve original x
	rts		; and back to DO_SND
	

; **** OUTPUT A WELCOME BANNER BEFORE EXECUTION (DHH)

WELCOM	LDA #$27		; code for dark grn-ylw
	STA $FF19		; TED border color
	LDA #$9E		; PETSCII code for yellow text
	JSR SAVOUT		; print it
	LDX #$00
	STX $FF15		; TED bkgrnd color to black
PRNT2	LDA TITLE,X		; print header (00 is EOL)
	BEQ SEED_P		; done, go seed P-RND generator
	JSR SAVOUT		; call out to screen routine
	INX			; +1
	BNE PRNT2		; loop
SEED_P	LDX #$05	; loop 6x
LOOP3	LDA TIMER	; get a byte from TED timer 3
	AND #$77	; strip high bit of both nybbles because
OK_99	STA RNUMB,X	;   dealing with decimal #s for P:
	DEX		; store and -1 X
	BPL LOOP3	; <=0 go do more, otherwise...
WELDUN	RTS		; go start TP proper

COLORS
.BYTE	$92 $93 $05 $1C $9F $9C $1E $1F $9E $12

TITLE
.BYTE	$93 $0D
.BYTE	"   6502 TINY PILOT FOR C16 AND PLUS/4"
.BYTE	$0D $00
.BYTE	"ORIG: NICHOLAS VRTIS, 1979"
.BYTE	" THIS PORT: DAVE HASSLER, 2025"
